@@ -39,18 +39,37 @@ getGradients.sgmcmc = function( sgmcmc ) {
     return( estLogPostGrads )
 }
 
-# Get gradient estimates for control variate methods
+# Get gradient estimates for control variate methods, handle IndexedSlices gradients gracefully
 getGradients.sgmcmccv = function( sgmcmcCV ) {
     estLogPostGrads = list()
     for ( pname in names( sgmcmcCV$params ) ) {
         gradCurr = tf$gradients( sgmcmcCV$estLogPost, sgmcmcCV$params[[pname]] )[[1]]
+        isSparse = gradIsIndexed(gradCurr)
         optGradCurr = tf$gradients( sgmcmcCV$estLogPostOpt, sgmcmcCV$paramsOpt[[pname]] )[[1]]
         optGradFull = sgmcmcCV$logPostOptGrad[[pname]]
-        estLogPostGrads[[pname]] = optGradFull - optGradCurr + gradCurr
+        if (isSparse) {
+            # Get the current gradient estimate but ensure to keep it as IndexedSlices object
+            fullGradCurr = tf$gather(optGradFull, gradCurr$indices)
+            currentVals = fullGradCurr - optGradCurr$values + gradCurr$values
+            estLogPostGrads[[pname]] = tf$IndexedSlices(currentVals, gradCurr$indices)
+        } else {
+            estLogPostGrads[[pname]] = optGradFull - optGradCurr + gradCurr
+        }
     }
     return( estLogPostGrads )
 }
 
+# Check if gradCurr is IndexedSlices object for minibatched parameters
+gradIsIndexed = function(grad) {
+    isSparse = tryCatch({
+        temp = grad$indices
+        TRUE
+    }, error = function (e) { 
+        return(FALSE)
+    })
+    return(isSparse)
+}
+
 # Calculate size of dataset
 getDatasetSize = function( data ) {
     N = dim( data[[1]] )[1]
@@ -150,5 +169,5 @@ convertList = function( tuningConst, params ) {
 
 # If tf$float64 error encountered, provide a more useful message
 throwFloat64Error = function( e ) {
-    stop(paste0("Problem building log posterior estimate from supplied logLik and logPrior functions. This can sometimes be due to constants declared in these functions resulting in objects of type tf$float64. Full output below should make it clear if this is the issue.\n\n", e))
+    stop(paste0("Problem building log posterior estimate from supplied logLik and logPrior functions. This is usually due to one of two things: a problem with the TensorFlow code used to declare logLik or logPrior functions; or that some constants declared in this function are read as type tf$float64. Maybe try to specify all constants as tf$float32. Full TensorFlow output below should make it clear if the issue is a float64 issue, or just a generic TensorFlow code issue.\n\n", e))
 }
